package ucot.model;

import java.io.Serializable;

/**
 * <pre>public class <b>Updation</b></pre>
 * <p>Class contains updation information about the model.
 * It contains the type of change and information about the
 * change.</p>
 * <p>
 * For example deletion of method would look like this:
 * </p>
 * <pre>new Updatition(UpdationType.DELETION, UpdatioDatatype.METHOD, "entity", "method");</pre
 * <p>It can be used to signal that the "method" of "entity" is deleted.</p> 
 *  
 * @author pajumasu
 */
public class Updation implements Serializable{
	public static final long serialVersionUID = 121289714289124L;
	/**
	 * Contains information about the
	 * updation type.
	 * @author pajumasu
	 *
	 */
	public enum UpdationType{
		/**
		 * Modification occurred.
		 */
		MODIFY,
		/**
		 * Cration ocurred.
		 */
		NEW,
		/**
		 * Deletion ocurred.
		 */
		DELETION,
		
		/**
		 * Model has just been completely wiped out.
		 */
		CLEAR,
		
		/**
		 * Big chunk of modification is done.
		 */
		READY,
		/**
		 * Big chunk of modification started. Time intensive
		 * updations should be cased and wait for ready signal.
		 */
		MODIFICATION_STARTED};
	
		/**
		 * Contains information about the type beign changed.
		 * @author pajumasu
		 */
		public enum UpdationDataType{
			/**
			 * Entity is beign altered.
			 * Reference contains the name of the entity.
			 */
			ENTITY, 
			/**
			 * Method is beign altered.
			 * Reference contains the name of the entity and the name of the method.
			 */
			METHOD,
			/**
			 * Attribute is beign altered.
			 * Reference contains the name of the entity and the name of the attribute. 
			 */
			ATTRIBUTE,
			/**
			 * Attribute to cardinality is beign altered.
			 * Reference contains the name of the entity and the name of the attribute
			 * and the to cardinality.
			 */
			ATTRIBUTE_TO_CARDINALITY,
			/**
			 * Attribute to cardinality is beign altered.
			 * Reference contains the name of the entity and the name of the attribute
			 *  and the from cardinality.
			 */
			ATTRIBUTE_FROM_CARDINALITY,
			/**
			 * Method influence is beign altered.
			 * Reference contains the name of the entity and the name of the methdod
			 *  and the name of the entity that is influenced.
			 */
			METHOD_INFLUENCE,
			/**
			 * Parent relation is beign altered.
			 * Reference contains the name of the entity which parent is changed
			 * and the entity that is set to the first ones parent.
			 */			
			ENTITY_PARENT,
			/**
			 * The type is beign altered
			 * Reference contains the name of the entity which type is changed
			 * and the type.
			 */			
			ENTITY_TYPE,
			/**
			 * No data caried.
			 */						
			EMPTY
		};
		
		
		
	UpdationType type;
	UpdationDataType dataType = UpdationDataType.EMPTY;
	String[] references = new String[0];
	
	
	public Updation(UpdationType type, UpdationDataType dataType){
		this.type = type;
		this.dataType = dataType;
	}
	
	public Updation(UpdationType type, UpdationDataType dataType, String... references){
		this.type = type;
		this.dataType = dataType;
		this.references = references;
	}

	public Updation(UpdationType type){
		this.type = type;
	}
	

	public UpdationDataType getDataType() {
		return dataType;
	}

	public String[] getReferences() {
		return references;
	}

	public UpdationType getType() {
		return type;
	}
	
}
