package ucot.parser;

import java.util.List;
import java.util.Vector;

import ucot.input.UseCase;

/**
 * Parsed data is returned by parser. This contains a list of sentences 
 * (containing words) that have been parsed from usecase.
 * @author pajumasu
 *
 */
public class ParsedData {

	/**
	 * Sentences parsed from usecase
	 */
    private List<Sentence> sentences = new Vector<Sentence>();
    
    /**
     * Usecase this ParsedData was created from
     */
    private UseCase usecase = null;
    
    public void setUseCase(UseCase a) {
    	usecase = a;
    }
    
    /**
     * return the usecase that this parsedData was created from
     * @return the usecase that this parsedData was created from
     */
    public UseCase getUseCase() {
    	return usecase;
    }
    
    public ParsedData() {}
    
    /**
     * Adds a new sentence
     * @param sentence sentence to add
     */
    public void addSentence(Sentence sentence) {
        sentences.add(sentence);
    }
    
    /**
     * Returns a list of sentences this parsedData contains
     * @return a list of sentences this parsedData contains
     */
    public List<Sentence> getSentences(){
        Vector<Sentence> ret = new Vector<Sentence>();
        ret.addAll(sentences);
        return ret;
    }
    

    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("ParsedData:\n");
        for(Sentence sentence : sentences){
            buffer.append(sentence);
            buffer.append("\n");
        }
        return buffer.toString();
    }

    
    /**
     * Returns the text of the original use case.
     * @return text of the original use case
     */
    public String getOriginal(){
            StringBuffer buffer = new StringBuffer();
            for(Sentence sentence : sentences){
                for(Word word : sentence.getWords()){
                    buffer.append(word.getOriginal());
                    buffer.append(" ");
                }
                buffer.append("\n");
            }
            return buffer.toString();

    }

}
