package ucot.ui;

import ucot.ModulePropertyInterface;
import ucot.core.ControlInterface;



//TODO comment the general usage of this interface better
//and other related tasks that the interface has to do

/**
 * Interface for the UI. A means for the core component to notify UI
 * that certain actions have been taken. For example file has been loaded
 * @author vevijopi
 *
 */
public interface UIInterface extends ModulePropertyInterface {
	/**
	 * Core signals user interface that analyze model has been exported
	 *
	 */
	public void exportDone();
	
	/**
	 * Core signals user interface that analyzemodel has been loaded
	 *
	 */
	public void analyzeModelLoaded();
	
	/**
	 * Core signals user interface that usecases have been loaded from file
	 *
	 */
	public void useCasesLoaded();
	
	/**
	 * Core signals user interface that usecases have been parsed,
	 * ran heuristic on and been added to given analyze model
	 */
	public void useCaseAdded();
	
	/**
	 * Set control interface
	 * @param a control interface (UCOT core)
	 */
	public void setControlInterface(ControlInterface a);
	
	/**
	 * Prints error message to the user.
	 * @param errorMessage Description of the error.
	 * @param errorTitle Title of the dialog
	 */
	public void printError(String errorMessage, String errorTitle);
	public void printError(String errorMessage);
	
	/**
	 * Shows user a dialog asking a question
	 * @param title title of the question dialog 
	 * @param question question to ask
	 * @param options options for the dialog
	 * @return User's choice (e.g. button)
	 */
	/*public Object questionDialog(String question, String title, int options);
	public Object questionDialog(String question, String title);
	public Object questionDialog(String question);*/
	
	/**
	 * Prints a warning to the user.
	 * @param warningMessage Description of the warning.
	 * @param warningTitle Title of the dialog.
	 */
	public void printWarning(String warningMessage, String warningTitle);
	public void printWarning(String warningMessage);
	
	/**
	 * Method for getting a new progressbar for showing the current progress
	 * status to the user and halting all other usage of the model editor.
	 * @return ProgressBar interface to the progress bar.
	 */
	public ProgressBarInterface getProgressBar();
}
