package ucot.ui.gui.tree.analyzemodeltree;

import java.awt.Component;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Set;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import ucot.core.ControlInterface;
import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.model.object.ObjectAnalyzeModel;
import ucot.ui.gui.GraphicalUI;
import ucot.ui.gui.dialog.EntityPropertiesDialog;
import ucot.ui.gui.dialog.MergeEntitiesDialog;
import ucot.ui.gui.dot.DotPanel;

//TODO comment me
public class AnalyzeModelTree extends JTree implements DragGestureListener, DropTargetListener {
	
	// TreeSelectionListener for reacting when user selects node
	private TreeSelectionListener treeSelectionListener;
	
	// who is notified when user selects a node..
	private DotPanel panel = null;
	private GraphicalUI owner;
	
	private DragSource dragSource;
	private DropTargetListener dtListener;
	private DropTarget dropTarget;
	private AnalyzeTreeModel atm;
	private AnalyzeModel analyzemodel = null;
	
    //
    // Icons for treeitems
    protected static Icon entitiesIcon  = new ImageIcon("img/icon/entities_icon.gif");
    protected static Icon entityIcon    = new ImageIcon("img/icon/entity_icon.gif");
    protected static Icon methodIcon    = new ImageIcon("img/icon/method_icon.gif");  
    protected static Icon attributeIcon = new ImageIcon("img/icon/attribute_icon.gif");
    protected static Icon childIcon     = new ImageIcon("img/icon/child_icon.gif");

    
    /**
     * Custom TreeCellRenderer for rendering icons for treeitems.
     * @author ilanliuk
     *
     */
    private class AnalyzemodelTreeCellRenderer extends DefaultTreeCellRenderer {
        
        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
                    row, hasFocus);
         
            // get Icon for this treeitem
            if (value instanceof TreeItem) {
                TreeItem item = (TreeItem) value;
                setIcon(item.getIcon());
            }
            
            return this;
        }
    }
    
    /**
     * Creates MouseListener for AnalyzeModelTree to handle MouseClickedEvent over the tree
     * @param core ControlInterface
     * @param entityPropertiesDialog EntityPropertiesDielog for editing entity properties.
     */
	public void createMouseListener(final ControlInterface core, final EntityPropertiesDialog entityPropertiesDialog) {
		addMouseListener(new MouseAdapter() {
			
			public void mouseClicked(MouseEvent e) {
				// See if we cliked to any node in the tree.
				final TreePath selectionPath = getPathForLocation(e.getX(), e.getY());
				if (selectionPath != null){
					final TreeItem selected = (TreeItem) selectionPath.getLastPathComponent();
					
					
					// If user clicked second mouse button
					// (MouseEvent.BUTTON2 is scroll)
					if (e.getButton() == MouseEvent.BUTTON3) {
						
						// Set selection right.
						setSelectionPath(selectionPath);
						
						// Creating popup if something is selected.
						if (selectionPath != null) {
							JPopupMenu menu = new JPopupMenu();
							
							
                            //
                            // if selected is EntitiesTreeItem add choice 
                            // to add new entity into model.
							if (selected instanceof EntitiesTreeItem) {
							    menu.add( getNewEntityMenuItem() );
							}
                            
                            
                            //
                            // in case of selected is AttributeTreeItem 
                            // add choices
                            //   - to edit parent of attribute
                            //   - to edit this entity
							//   - to merge this entity
                            //   - to remove attribute
                            //   - to delete this entity
							if (selected instanceof AttributeTreeItem) {

                                String entity = selected.getName();
                                String parent = selected.getParent().getName();
                                
                                menu.add( getModifyEntityMenuItem(parent,EntityPropertiesDialog.ATTRIBUTES_TAB_KEY) );
                                menu.add( getModifyEntityMenuItem(entity));
                                
                                menu.addSeparator();

                                menu.add( getMergeDialogMenuItem(entity) );
									
                                menu.addSeparator();

                                menu.add( getRemoveAttributeMenuItem(parent, entity) );
								menu.add( getDeleteEntityMenuItem(entity) );
                            }
			
                            
                            //
                            // in case of selected is ChildTreeItem 
                            // add choices
                            //   - to edit parent of this
                            //   - to edit this entity
                            //   - to merge this entity
                            //   - to remove child
                            //   - to delete this entity
							if (selected instanceof ChildTreeItem) {
                                
                                String entity = selected.getName();
                                String parent = selected.getParent().getName();
                                
                                menu.add( getModifyEntityMenuItem(parent,EntityPropertiesDialog.CHILDREN_TAB_KEY ));
                                menu.add( getModifyEntityMenuItem(entity) );
                                
                                menu.addSeparator();

                                menu.add( getMergeDialogMenuItem(entity) );

                                menu.addSeparator();

                                menu.add( getRemoveChildMenuItem(parent, entity) );
                                menu.add( getDeleteEntityMenuItem(entity) );
                            
                            }
                            
                            //
                            // if selected is EntityTreeItem and is located
                            // just below EntitiesTreeItem add choices:
                            //   - to edit this entity
                            //   - to merge this entity
                            //   - to delete this entity
							if (selected instanceof EntityTreeItem 
                                    && selected.getParent() instanceof EntitiesTreeItem) {

                                String entity = selected.getName();

                                menu.add( getModifyEntityMenuItem(entity) );
                                
                                menu.addSeparator();

                                menu.add( getMergeDialogMenuItem(entity) );
                                
                                menu.addSeparator();

                                menu.add( getDeleteEntityMenuItem(entity) );
							}
							
                            
                            //
                            // if selected is MethodTreeItem add choice
                            //   - to modify owner of this method 
                            //   - to remove this method
                            if (selected instanceof MethodTreeItem) {
                                menu.add( getModifyEntityMenuItem(selected.getParent().getName()) );

                                menu.addSeparator();
                                
                                menu.add( getRemoveMethodMenuItem(selected.getParent().getName(), selected.getName()));
                                
                            }

                            
                            // 
                            // if selected EntityTreeItem is method target
                            // add choices:
                            //   - to edit entity owning method 
                            //   - to edit this entity
                            //   - to merge this entity
                            //   - to change this entity from method target to attribute
                            //   - to change this entity from method target to child
                            //   - to remove method relation
                            //   - to remove this entity
							if (selected instanceof EntityTreeItem
									&& selected.getParent() instanceof MethodTreeItem) {

                                String entity = selected.getName();
                                String method = selected.getParent().getName();
                                String parent = selected.getParent().getParent().getName();
                                
                                menu.add( getModifyEntityMenuItem(parent,EntityPropertiesDialog.METHODS_TAB_KEY ));
                                menu.add( getModifyEntityMenuItem(entity) );
                                
                                
                                menu.addSeparator();

                                menu.add( getMergeDialogMenuItem(entity) );
                                menu.add( getChangeFromMethodToAttributeMenuItem(parent, method, entity) );
                                
                                // to make sure user doesn't add entity 
                                // as its own child.
                                if (!entity.equals(parent)) {
                                	menu.add( getChangeFromMethodToChildMenuItem(parent, method, entity) );
                                }

                                menu.addSeparator();

                                menu.add( getRemoveMethodTargetMenuItem(parent, method, entity) );
								menu.add( getDeleteEntityMenuItem(entity) ); 
                            }
							
							// Show popup menu relative to the components
							// coordinates.
							// This shows popup exactly where user pressed.
							menu.show(e.getComponent(), e.getX(), e.getY());
						}
					}
				}
			}
            
            /**
             * Returns MenuItem for adding new entity
             * @return JMenuItem
             */
            private JMenuItem getNewEntityMenuItem() {
                JMenuItem result = new JMenuItem("New entity..");
                result.addActionListener( new ActionListener() {
                
                    public void actionPerformed(ActionEvent arg0) {

                        // Add entity called "new entity" to
                        // analyzeModel and
                        // then open modifying dialog with it..
                        
                        String entityName = "New entity";
                        int count = 0;
                        
                        while ( core.getAnalyzeModel().getEditor().containsEntity(entityName) ) {
                            count++;
                            entityName = "New entity" + count;
                        }
                        
                        entityPropertiesDialog.modifyEntityProperties(entityName, core.getAnalyzeModel());
                    }
                
                } );
                return result;
            }
            
            /**
             * Returns Modify entity-MenuItem for given entity and opens given tab.
             * @param entity Name of the entity.
             * @param tab The key of the tab to be opened.
             * @return JMenuItem
             */
            private JMenuItem getModifyEntityMenuItem(final String entity, final String tab) {
                JMenuItem result = new JMenuItem("Modify " + entity + "..");
                result.addActionListener( new ActionListener() {
                
                    public void actionPerformed(ActionEvent e) {
                        entityPropertiesDialog.modifyEntityProperties(entity, core.getAnalyzeModel());
                        entityPropertiesDialog.showTabFor(tab);
                    }
                } );
                return result;
            }
            /**
             * Returns Modify entity-MenuItem for given entity.
             * @param entity Name of the entity.
             * @param tab The key of the tab to be opened.
             * @return JMenuItem
             */            
            private JMenuItem getModifyEntityMenuItem(final String entity) {
            	return getModifyEntityMenuItem(entity,null);
            }
            
            /**
             * Returns Delete entity -MenuItem for given entity
             * @param entity Name of the entity
             * @return JMenuItem
             */
            private JMenuItem getDeleteEntityMenuItem(final String entity) {
                JMenuItem result = new JMenuItem("Delete entity");
                result.addActionListener( new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        core.getAnalyzeModel().getEditor().removeEntity(entity);
                    }
                } );
                return result;
            }
            
            /**
             * Returns Remove attribute -MenuItem for given entity and attribute.
             * @param entity Name of the entity.
             * @param attribute Name of the attribute.
             * @return JMenuItem
             */
            private JMenuItem getRemoveAttributeMenuItem(final String entity, final String attribute) {
                JMenuItem result = new JMenuItem("Remove attribute");
                result.addActionListener( new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        core.getAnalyzeModel().getEditor().removeAttribute(entity, attribute);
                    }
                } );
                return result;
            }
            
            /**
             * Returns Remove method -MenuItem for given entity and method.
             * @param entity Name of the entity.
             * @param method Name of the method.
             * @return JMenuItem
             */
            private JMenuItem getRemoveMethodMenuItem(final String entity, final String method) {
                JMenuItem result = new JMenuItem("Remove method");
                result.addActionListener( new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        core.getAnalyzeModel().getEditor().removeMethod(entity, method);
                    }
                } );
                return result;
            }

            /**
             * Returns Remove child -MenuItem for given parent and child entities.
             * @param parent Name of the parent entity.
             * @param child Name of the child entity.
             * @return JMenuItem
             */
            private JMenuItem getRemoveChildMenuItem(final String parent, final String child) {
                JMenuItem result = new JMenuItem("Remove child");
                result.addActionListener( new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        core.getAnalyzeModel().getEditor().removeChild(parent, child);
                    }
                } );
                return result;
            }

            /**
             * Get MenuItem for changing method influence into attribute influence.
             * @param parent parent entity
             * @param method influencing method
             * @param entity target entity
             * @return JMenuItem
             */
            private JMenuItem getChangeFromMethodToAttributeMenuItem(final String parent, final String method, final String entity) {
                JMenuItem result = new JMenuItem("Change from method to attribute");
                result.addActionListener( new ActionListener() {
                
                    public void actionPerformed(ActionEvent e) {
                        ModelEditor editor = core.getAnalyzeModel().getEditor();
                        
                        // remove entity from list of entities influenced by this method
                        editor.removeEntityInfluenceByMethod(parent, method, entity);

                        // if method influenced nothing but this entity, remove it
                        Set<String> targets = editor.getEntitiesInfluencedByMethod(parent, method);
                        if (targets.size() == 0) {
                            editor.removeMethod(parent, method);
                        }
                        
                        // add entity as an attribute of the parent entity
                        editor.addAttribute(parent, entity);
                    }
                
                } );
                return result;
            }
            
            /**
             * Get MenuItem for changing method influence into inheritance influence.
             * @param parent parent entity
             * @param method influencing method
             * @param entity target entity
             * @return JMenuItem
             */
            private JMenuItem getChangeFromMethodToChildMenuItem(final String parent, final String method, final String entity) {
                JMenuItem result = new JMenuItem("Change from method to child");
                result.addActionListener( new ActionListener() {
                
                    public void actionPerformed(ActionEvent e) {
                        ModelEditor editor = core.getAnalyzeModel().getEditor();
                        
                        // remove entity from list of entities influenced by this method
                        editor.removeEntityInfluenceByMethod(parent, method, entity);

                        // if method influenced nothing but this entity, remove it
                        Set<String> targets = editor.getEntitiesInfluencedByMethod(parent, method);
                        if (targets.size() == 0) {
                            editor.removeMethod(parent, method);
                        }
                        
                        // add entity as an attribute of the parent entity
                        editor.addChild(parent, entity);
                    }
                
                } );
                return result;
            }

            /**
             * Returns MenuItem for launching merge-dialog for given entity
             * @param entity entity to merge.
             * @return JMenuItem
             */
            private JMenuItem getMergeDialogMenuItem(final String entity) {
                JMenuItem result = new JMenuItem("Merge " + entity + " with..");
                result.addActionListener( new ActionListener() {
                
                    public void actionPerformed(ActionEvent arg0) {
                        MergeEntitiesDialog.showDialog(core, entity);
                    }
                } );
                return result;
            }

            /**
             * Returns Remove method target -MenuItem for given entity influenced bt method.
             * @param entity Name of the entity that owns the method.
             * @param method name of method.
             * @param influencedEntity entity to remove from method targets.
             * @return
             */
            private JMenuItem getRemoveMethodTargetMenuItem(final String entity, final String method, final String influencedEntity) {
                JMenuItem result = new JMenuItem("Remove method target");
                result.addActionListener( new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        core.getAnalyzeModel().getEditor().removeEntityInfluenceByMethod(entity, method, influencedEntity);
                    }
                
                } );
                return result;
            }
		});
	}
	
	public AnalyzeModelTree(GraphicalUI owner) {
		this.owner = owner;
		
		//this.setDragEnabled(true);
		this.dragSource = DragSource.getDefaultDragSource();
		this.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		
		
		this.dragSource.createDefaultDragGestureRecognizer(
				this, DnDConstants.ACTION_MOVE, this );
		
		atm = new AnalyzeTreeModel(owner);
		
		this.setModel(atm);
		this.dtListener = this;
		
		// component, ops, listener, accepting
		this.dropTarget = new DropTarget(this,
				this.acceptableActions,
				this.dtListener,
				true);
		
		// create the TreeSelectionListener and override selection event..
		treeSelectionListener = new TreeSelectionListener() {
			
			public void valueChanged(TreeSelectionEvent e) {
				
				TreeItem item = (TreeItem) e.getPath().getLastPathComponent();
				
				// if dotpanel is assigned..
				if (panel == null) return;
				if (item == null) return;
				
				AnalyzeModel highlightModel = new ObjectAnalyzeModel();
				if ( item instanceof EntityTreeItem ) {
					highlightModel.getEditor().addEntity(item.getName());
				}
				if ( item instanceof MethodTreeItem ) {
					highlightModel.getEditor().addEntity(item.getParent().getName());
					Set<String> targets = atm.getAnalyzeModel().getEditor().
					getEntitiesInfluencedByMethod(item.getParent().getName(), item.getName());
					highlightModel.getEditor().addMethod(item.getParent().getName(), item.getName());
					for ( String target: targets ) {
						highlightModel.getEditor().addEntity(target);
						highlightModel.getEditor().addEntityInfluenceByMethod(
								item.getParent().getName(), item.getName(), target);
					}
				}

				if ( item instanceof AttributeTreeItem ) {
					highlightModel.getEditor().addEntity(item.getParent().getName());
					highlightModel.getEditor().addEntity(item.getName());
					highlightModel.getEditor().addAttribute(item.getParent().getName(), item.getName());
				}
				
				
				panel.highlight(highlightModel);
			}
			
		};
		this.addTreeSelectionListener( treeSelectionListener );
		
        
        // Initialize TreeCellRenderer for AnalyzeModelTree
        this.setCellRenderer(new AnalyzemodelTreeCellRenderer());
	}
	
	private int acceptableActions = DnDConstants.ACTION_COPY;
	
	public void dragGestureRecognized(DragGestureEvent dragEvent) {
		TreePath path = getSelectionModel().getSelectionPath();
		if (null != path){
			TreeItem treeItem = (TreeItem) path.getLastPathComponent();
			/*Object data = treeItem.getData();
			 if (data instanceof Entity){
			 dragEvent.startDrag(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR), 
			 new DataTransfer(data));
			 }*/
			
		}
		
	}
	
	public void dragEnter(DropTargetDragEvent arg0) {
		System.out.println(arg0);
	}
	
	public void dragExit(DropTargetEvent arg0) {
		System.out.println(arg0);
		
	}
	
	public void dragOver(DropTargetDragEvent arg0) {
		
		
	}
	
	public void drop(DropTargetDropEvent dtde){
		
		//The Drag operation has terminated with a Drop on this DropTarget
		Transferable transferable = dtde.getTransferable();
		
	}
	
	public void printNodeContents(TreeNode parent)
	{
		System.out.println("Node is " + parent);
		for(int i = 0 ; i<parent.getChildCount(); i++){
			System.out.println(parent.getChildAt(i));
		}
	}
	public void dropActionChanged(DropTargetDragEvent arg0) {
		System.out.println(arg0);
		
	}
	
    
	public void setStructurePanel(DotPanel dotPanel) {
		this.panel = dotPanel;
	}
	
	
}
