package ucot.ui.gui.tree.usecasetree;

import java.util.List;
import java.util.Vector;

import javax.swing.Icon;

import ucot.input.UseCaseCollection;

/**
 * <pre>public abstract class <b>TreeItem</b></pre>
 * 
 * <p><code>Abstract</code> superclass for all <code>UseCaseTree</code>s tree nodes.</p>
 * 
 * @see #ucot.ui.gui.tree.usecasetree.UseCaseTree
 * 
 * @author ilanliuk
 *
 */
public abstract class TreeItem {
    
    Object data;
    String name;
    UseCaseTreeModel treemodel;
    TreeItem parent;

    /**
     * <p>Constructor for TreeItem.</p>
     * 
     * @param treemodel <code>UseCaseTreeModel</code>
     * @param parent <code>TreeItem</code> wich is parent of this <code>TreeItem</code>.
     */
    public TreeItem(UseCaseTreeModel treemodel, TreeItem parent) {
        this.treemodel = treemodel;
        this.parent = parent;
    }
    
    /**
     * <p>Returns treemodel.</p>
     * 
     * @return <code>UseCaseTreeModel</code>
     */
    public UseCaseTreeModel getTreeModel() {
        return this.treemodel;
    }
    
    /**
     * <p>Returns <code>UseCaseCollection</code> of the <code>UseCaseTreeModel</code>.</p>
     * 
     * @return <code>UseCaseCollection</code>
     */
    public UseCaseCollection getUseCaseCollection() {
        return this.treemodel.getUseCaseCollection();
    }

    /**
     * <p>Returns name of this item.</p>
     * 
     * @return <code>String</code> name.
     */
    public abstract String getName();
 
    /**
     * <p>Returns data contained by this TreeItem.</p>
     * 
     * @return Data of this <code>TreeItem</code> as <code>Object</code>.
     */
    public Object getData() {
        return this.getName();
    }
    
    public String toString() {
        return this.getName();
    }
    
    /**
     * <p>Return children of this <code>TreeItem</code> 
     * in a <code>List</code>.</p>
     * 
     * @return <code>List</code> of children.
     */
    public List<TreeItem> getChildren() {
        return new Vector<TreeItem>();
    }
    
    /**
     * <p>Returns path to this item as a <code>List</code> of <code>TreeItem</code>s.</p>
     * 
     * @return <code>List</code> of items in the path.
     */
    public List<TreeItem> getPath() {
        List<TreeItem> path = new Vector<TreeItem>();
        path.add(0, this);
        if (parent != null) {
            path.addAll(0, parent.getPath());
        }
        return path;
    }
    
    /**
     * <p>Returns is this <code>TreeItem</code> leaf-node or not.</p>
     * 
     * @return is this <code>TreeItem</code> leaf-node or not
     */
    public boolean isLeaf() {
        return (this.getChildren().size() == 0);
    }
    
    /**
     * <p>Returns <code>Icon</code> for this <code>TreeItem</code>.</p>
     * 
     * @return <code>Icon</code> for this <code>TreeItem</code>.
     */
    public abstract Icon getIcon();
}
